(********************************************************)
(*                   WindowHelp2.mod                    *)
(* Module mit Hilfen fr Windows,Gadgets,Screens Images *)
(* (C) 1988 Frank Schfer & Carsten Manns & TOOLBOX     *)

IMPLEMENTATION MODULE WindowHelp2;

FROM Arts       IMPORT Assert;
FROM Exec       IMPORT CopyMem,GetMsg,MemReqs,ReplyMsg,
 TypeOfMem,UByte,WaitPort,MemReqSet;
FROM Graphics   IMPORT ViewModes,ViewModeSet;
FROM Intuition  IMPORT ActivationFlags,ActivationFlagSet,
 boolGadget,CloseWindow,customScreen,Gadget,
 GadgetFlags,GadgetFlagSet,GadgetPtr,IDCMPFlagSet,Image,
 ImagePtr,IntuiMessage,IntuiMessagePtr,IntuitionBase,
 NewScreen,NewWindow,OpenIntuition,OpenScreen,OpenWindow,
 RefreshGadgets,RefreshGList, RememberPtr,ScreenPtr,
 ScreenFlags,ScreenFlagSet,Window,WindowFlagSet,WindowPtr,
 IntuiTextPtr;
FROM SYSTEM     IMPORT ADR,ADDRESS,INLINE,LONGSET,CAST;
FROM Heap  IMPORT AllocMem;

CONST MessageRam = "Kein Speicher frei";

(*-----------------------------------------------------*)
(* Erzeugen einer Speicherliste, genau wie bei der     *)
(* Funktion AllocRemember aus Intuition. Unter Kick    *)
(* 1.3 traten allerdings Probleme bei AllocRemember    *)
(* und FreeRemember auf.                               *)
(* Freigeben des Speichers siehe File.mod              *)
PROCEDURE MakeList (VAR FirstPtr : RememberPtr;
                        Size : LONGCARD;
                        Chip : BOOLEAN) : ADDRESS;

VAR  Element : ADDRESS;
     i       : RememberPtr;

BEGIN
 AllocMem (Element,Size,Chip);
 Assert (Element # NIL,ADR (MessageRam));
 IF FirstPtr = NIL THEN
   AllocMem (FirstPtr,SIZE (FirstPtr^),FALSE);
   Assert (FirstPtr # NIL,ADR (MessageRam));
   FirstPtr^.memory := Element;
   FirstPtr^.rememberSize := Size;
   FirstPtr^.nextRemember := NIL
 ELSE
   AllocMem (i,SIZE (i^),FALSE);
   Assert (i # NIL,ADR (MessageRam));
   i^.memory := Element;
   i^.rememberSize := Size;
   i^.nextRemember := FirstPtr
 END;
 RETURN Element
END MakeList;


(*-----------------------------------------------------*)
(* Standartprocedure zum Erzeugen eines Gadgets.       *)
(* Wie man sie anwendet sieht man in dem MODULE        *)
(* SmallWindow2, oder in WindowStuff.                  *)
PROCEDURE GADGET (VAR NewGadget : Gadget;
                  X, Y, W, H : INTEGER;
                  GFlags : GadgetFlagSet;
                  AFlags : ActivationFlagSet;
                  I1, I2 : ImagePtr;
                  Text : IntuiTextPtr;
                  Nummer : INTEGER);

BEGIN
  WITH NewGadget DO
   gadgetRender  := I1;         gadgetText   := Text;
   nextGadget    := NIL;        leftEdge     := X;
   topEdge    := Y;          width        := W;
   height    := H;          flags        := GFlags;
   activation    := AFlags;     selectRender := I2;
   gadgetType    := boolGadget; specialInfo  := NIL;
   mutualExclude := LONGSET{0}; userData     := NIL;
   gadgetID     := Nummer
  END
END GADGET;

(*------------------------------------------------------*)
(* Aus einer INLINE-Procedure mit Grafikdaten ein Image *)
(* machen. Start ist die Adresse der INLINE-Procedure,  *)
(* wenn die Grafikdaten im FAST-Memory liegen werden    *)
(* Sie ins CHIP-Mem kopiert.                            *)
PROCEDURE MakeImage (Start : ADDRESS; VAR Grafik : Image;
         PlaneP,PlaneO : UByte; VAR RamPtr : RememberPtr);

TYPE  IMAGEDatenPtr = POINTER TO IMAGEDaten;
      IMAGEPtr      = POINTER TO ARRAY [0..50] OF CARDINAL;
      IMAGEDaten    = RECORD
          Groesse,x,y,depth : CARDINAL
               END;

VAR i    : CARDINAL;
    IDatenPtr  : IMAGEDatenPtr;
    Daten       : ADDRESS;

BEGIN
  IDatenPtr := Start;
  INC(Start,16);
  IF fast IN TypeOfMem (Start) THEN
    i := (IDatenPtr^.Groesse + 1) * 2;
    Daten := MakeList (RamPtr,i,TRUE);
    CopyMem (Start,Daten,i)
  ELSE
    Daten := Start
  END;
  WITH IDatenPtr^ DO
    WITH Grafik DO
      leftEdge   := 0;       topEdge    := 0;
      width   := x;       height     := y;
      depth      := depth;   imageData  := Daten;
      planePick  := PlaneP;  planeOnOff := PlaneO;
      nextImage  := NIL
    END
  END
END MakeImage;

(*------------------------------------------------------*)
(* Die X- und Y-Position so berechnen das sich das      *)
(* Window unter der Maus ffnet. Dazu werden aus        *)
(* IntuitionBase die aktuellen Mausdaten besorgt.       *)
(* Der ScreenPointer auf den Screen mu schon           *)
(* eingetragen sein, geffnet mu der Screen nicht sein.*)
PROCEDURE NearMouse (VAR Window : NewWindow);

VAR IBasePtr : POINTER TO IntuitionBase;
    x,y,xMax,yMax,Faktor : INTEGER;

BEGIN
 IBasePtr := OpenIntuition ();
 WITH Window DO
   Faktor := 640 DIV screen^.width;
   IF Faktor = 0 THEN Faktor := 1 END;
   leftEdge := IBasePtr^.mouseX DIV Faktor - width DIV 2;
   Faktor := 512 DIV screen^.height;
   IF Faktor = 0 THEN Faktor := 1 END;
   topEdge := IBasePtr^.mouseY DIV Faktor - height DIV 2;
   IF leftEdge + width >= screen^.width THEN
      leftEdge := screen^.width - width
   ELSIF leftEdge < 0 THEN
      leftEdge := 0
   END;
   IF topEdge + height >= screen^.height THEN
      topEdge := screen^.height - height
   ELSIF topEdge < 0 THEN
      topEdge := 0
   END
 END
END NearMouse;

(*-------------------------------------------------------*)
(* Procedure zum ffnen eines Screens. Zurckgegeben     *)
(* wird der Screenpointer oder NIL.                      *)
PROCEDURE SCREEN (Top,Width,Height,Depth,APen,BPen
                  : INTEGER; Mode : ViewModeSet;
                  Name : ADDRESS; Vorne : BOOLEAN)
                  : ScreenPtr;

VAR Screen     : NewScreen;
    ScreenFlag : ScreenFlagSet;

BEGIN
 ScreenFlag:=customScreen;
  IF Width > 320 THEN
     INCL (Mode,hires)
 END;
 IF Height > 256 THEN        
    INCL (Mode,lace)
 END;
 IF NOT(Vorne) THEN
  INCL(ScreenFlag,screenBehind)
 END;
 WITH Screen DO
    leftEdge     := 0;          topEdge       := Top;
    width         := Width;      height        := Height;
    depth        := Depth;      detailPen     := APen;
    blockPen     := BPen;       viewModes     := Mode;
    type         := ScreenFlag; font          := NIL;
    defaultTitle := Name;       gadgets       := NIL;
    customBitMap := NIL
 END;
 RETURN OpenScreen (Screen)
END SCREEN;

(*-----------------------------------------------------*)
(* Grundprocedure zum Warten auf eine IntuiMessage.    *)
PROCEDURE WaitEvent (VAR WindowPointer : WindowPtr;
                     VAR Message : IntuiMessage);

VAR MsgPtr  : IntuiMessagePtr;

BEGIN
    WaitPort (WindowPointer^.userPort);
    MsgPtr := GetMsg (WindowPointer^.userPort);
    Message := MsgPtr^;
    ReplyMsg (MsgPtr)
END WaitEvent;

(*------------------------------------------------------*)
(* ffnen eines Windows auf einem Screen.               *)
(* Alle wichtigen Parameter knnen bergeben werden.    *)
(* Wenn man fr Left und Top -1 bergibt ffnet sich    *)
(* das Window unter der Maus.                           *)
PROCEDURE WINDOW (MyScreen : ScreenPtr;
                  X,Y,Left,Top : INTEGER;
                  BlockPen, DetailPen : UByte;
                  Title : ADDRESS;
                  WindowGadget : GadgetPtr;
                  WindowFlags : WindowFlagSet;
                  PortFlags : IDCMPFlagSet ) : WindowPtr;

VAR Window : NewWindow;
    WPtr   : WindowPtr;

BEGIN
 WITH Window DO
   leftEdge    := Left;         topEdge   := Top;
   width       := X;            height    := Y;
   blockPen    := BlockPen;     detailPen := DetailPen;
   title       := Title;        flags     := WindowFlags;
   idcmpFlags  := PortFlags;    checkMark := NIL;
   firstGadget := WindowGadget; checkMark := NIL;
   screen      := MyScreen;     type      := customScreen;
   bitMap       := NIL;         minWidth  := 0;
   minHeight   := 0;            maxWidth  := 0;
   maxHeight   := 0
 END;
 IF (Top = -1)  AND (Left = -1) THEN
   NearMouse (Window); (* Window unter der Maus ffnen *)
 END;
 WPtr := OpenWindow(Window);       
 RETURN WPtr
END WINDOW;

(*------------------------------------------------------*)

END WindowHelp2.
